@@ -1,5 +1,25 @@
 seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
     useExp.k = TRUE
+    search.min<-function(h, psi, psi.old) { #, mfExt , X, y, w, offs
+      psi.ok<- psi*h + psi.old*(1-h)
+      PSI <- matrix(rep(psi.ok, rep(n, length(psi.ok))), ncol = length(psi.ok))
+      U1 <- (Z - PSI) * (Z > PSI)
+      #if (pow[1] != 1) U1 <- U1^pow[1]
+      if(is.null(opz$mydesign)){
+        for (i in 1:ncol(U1)) {
+          mfExt[nomiU[i]] <- U1[, i]
+        }
+      } else {
+        for (i in 1:ncol(U1)) {
+          mydesign$variables[nomiU[i]] <- U1[, i]
+        }
+      }
+      obj1 <- suppressWarnings(try(eval(call.noV, envir = mfExt), silent = TRUE))
+      L1 <- if (class(obj1)[1] == "try-error") L0 + 10
+          else eval(parse(text = fn.obj), list(x = obj1))
+      L1
+    }
+    
     est.k <- function(x1, y1, L0) {
         ax <- log(x1)
         .x <- cbind(1, ax, ax^2)
@@ -15,8 +35,8 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
         else -pow * ((x - y) * (x > y))^(pow - 1)
     }
     in.psi <- function(LIM, PSI, ret.id = TRUE) {
-        a <- PSI[1, ] <= LIM[1, ]
-        b <- PSI[1, ] >= LIM[2, ]
+        a <- PSI[1, ] < LIM[1, ]
+        b <- PSI[1, ] > LIM[2, ]
         is.ok <- !a & !b
         if (ret.id) 
             return(is.ok)
@@ -74,16 +94,15 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
     if (sum(c1 + c2) != 0 || is.na(sum(c1 + c2))) 
         stop("psi out of the range")
     n <- nrow(Z)
-    min.step <- opz$min.step
+    #min.step <- opz$min.step
     rangeZ <- apply(Z, 2, range)
     alpha <- opz$alpha
-    limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha, 
-        1 - alpha))
+    limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha[1], alpha[2]))
     digits <- opz$digits
     pow <- opz$pow
     nomiOK <- opz$nomiOK
     toll <- opz$toll
-    h <- opz$h
+    hh <- opz$h
     conv.psi <- opz$conv.psi
     gap <- opz$gap
     stop.if.error <- opz$stop.if.error
@@ -94,6 +113,8 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
     it.max <- old.it.max <- opz$it.max
     fc<-opz$fc
     psi <- PSI[1, ]
+    psi<-adj.psi(psi, limZ)
+    PSI<-matrix(psi,nrow=n, ncol=ncol(PSI), byrow=TRUE)
     names(psi) <- id.psi.group
     epsilon <- 10
     dev.values <- psi.values <- NULL
@@ -107,15 +128,19 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
     call.ok <- opz$call.ok
     call.noV <- opz$call.noV
     toll <- opz$toll
+    mydesign<- opz$mydesign
+    
+    #browser()
+    
     if (!in.psi(limZ, PSI, FALSE)) 
         stop("starting psi out of the range", call. = FALSE)
     if (!far.psi(Z, PSI, id.psi.group, FALSE)) 
-        stop("psi values too close each other. Please change (decreases number of) starting values", 
-            call. = FALSE)
+      stop("psi starting values too close each other or at the boundaries. Please change them (e.g. set 'quant=TRUE' 
+          in seg.control()), or decrease their number.", call. = FALSE)
     n.psi1 <- ncol(Z)
     if (is.null(opz$constr)) 
         opz$constr <- 0
-    if ((opz$constr %in% 1:2) && class(obj) == "rq") {
+    if ((opz$constr %in% 1:2) && class(obj)[1] == "rq") {
         vincoli <- TRUE
         call.ok$method <- "fnc"
         call.ok$R <- quote(R)
@@ -127,14 +152,36 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
     fn.obj <- opz$fn.obj
     U <- ((Z - PSI) * (Z > PSI))
     colnames(U) <- nomiU
-    if (pow[1] != 1) 
-        U <- U^pow[1]
-    obj0 <- suppressWarnings(try(eval(call.noV, envir = mfExt), 
-        silent = TRUE))
+    #if (pow[1] != 1) U <- U^pow[1]
+    
+   
+    
+    obj0 <- suppressWarnings(try(eval(call.noV, envir = mfExt), silent = TRUE))
     if ("try-error" %in% class(obj0)) 
         stop("The first fit with U variables does not work..", 
             call. = FALSE)
     L0 <- eval(parse(text = fn.obj), list(x = obj0))
+    
+    
+    if(it.max==0){
+      colnames(U) <- paste("U", 1:ncol(U), sep = "")
+      V <- -(Z > PSI)
+      colnames(V) <- paste("V", 1:ncol(V), sep = "")
+      obj <- obj0 #lm.wfit(x = cbind(XREG, U), y = y, w = w, offset = offs)
+      L1 <- L0 #sum(obj$residuals^2 * w)
+      obj$coefficients <- c(obj$coefficients, rep(0, ncol(V)))
+      #names(obj$coefficients) <- names.coef
+      obj$epsilon <- epsilon
+      obj$it <- it
+      obj <- list(obj = obj, it = it, psi = psi, psi.values = psi.values, 
+                  U = U, V = V, rangeZ = rangeZ, epsilon = epsilon, nomiOK = nomiOK, 
+                  SumSquares.no.gap = L1, id.psi.group = id.psi.group, 
+                  id.warn = TRUE, nomiV = nomiV, nomiU = nomiU, mfExt = mfExt, 
+                  mydesign=mydesign)
+      return(obj)
+    }
+    
+
     n.intDev0 <- nchar(strsplit(as.character(L0), "\\.")[[1]][1])
     dev.values[length(dev.values) + 1] <- opz$dev0
     dev.values[length(dev.values) + 1] <- L0
@@ -149,22 +196,31 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
     }
     id.warn <- FALSE
     id.psi.changed<-rep(FALSE, it.max)
+    tolOp <-if(is.null(opz$tol.opt)) seq(.001, .Machine$double.eps^0.25, l=it.max) else rep(opz$tol.opt, it.max)
     while (abs(epsilon) > toll) {
         it <- it + 1
         n.psi0 <- n.psi1
         n.psi1 <- ncol(Z)
         if (n.psi1 != n.psi0) {
             U <- ((Z - PSI) * (Z > PSI))
-            if (pow[1] != 1) 
-                U <- U^pow[1]
-            obj0 <- suppressWarnings(try(eval(call.noV, envir = mfExt), 
-                silent = TRUE))
+            if (pow[1] != 1) U <- U^pow[1]
+            for (i in 1:ncol(U)) {
+              mfExt[nomiU[i]] <- U[, i]
+            }
+            obj0 <- suppressWarnings(try(eval(call.noV, envir = mfExt), silent = TRUE))
             L0 <- eval(parse(text = fn.obj), list(x = obj0))
         }
         V <- dpmax(Z, PSI, pow = pow[2])
-        for (i in 1:n.psi1) {
+        if(is.null(opz$mydesign)){
+          for (i in 1:n.psi1) {
             mfExt[nomiU[i]] <- U[, i]
             mfExt[nomiV[i]] <- V[, i]
+          }
+        } else {
+          for (i in 1:n.psi1) {
+            mydesign$variables[nomiU[i]] <- U[, i]
+            mydesign$variables[nomiV[i]] <- V[, i]
+          }
         }
         R <- fn.costr(ncol(U), 1, 1)
         R.noV <- R[, -((ncol(R) - 1) + seq_len(ncol(U))), drop = FALSE]
@@ -179,8 +235,7 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
                   return(list(dev.values, psi.values))
                 else stop("breakpoint estimate too close or at the boundary causing NA estimates.. too many breakpoints being estimated?", 
                   call. = FALSE)
-            }
-            else {
+            } else {
                 id.coef.ok <- !is.na(gamma.c)
                 psi <- psi[id.coef.ok]
                 gamma.c <- gamma.c[id.coef.ok]
@@ -194,50 +249,68 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
             }
         }
         psi.old <- psi
-        psi <- psi.old + gamma.c/beta.c
-        if (!is.null(digits)) 
-            psi <- round(psi, digits)
-        PSI <- matrix(rep(psi, rep(n, length(psi))), ncol = length(psi))
+        psi <- psi.old + hh*gamma.c/beta.c
+        psi<- adj.psi(psi, limZ)
+        
+        a<-optimize(search.min, c(0,1), psi=psi, psi.old=psi.old, tol=tolOp[it])
+        k.values[length(k.values) + 1] <- use.k <- a$minimum
+        L1<- a$objective
+        psi <- psi*use.k + psi.old* (1-use.k)
+        psi<- adj.psi(psi, limZ)
+        psi<-unlist(tapply(psi, opz$id.psi.group, sort), use.names =FALSE)
+        if (!is.null(digits)) psi <- round(psi, digits)
+        #PSI <- matrix(rep(psi, rep(n, length(psi))), ncol = length(psi))
+        PSI <- matrix(psi, ncol=length(psi), nrow=n, byrow = TRUE)
         U <- (Z - PSI) * (Z > PSI)
-        if (pow[1] != 1) 
-            U <- U^pow[1]
-        for (i in 1:ncol(U)) mfExt[nomiU[i]] <- U[, i]
-        obj1 <- suppressWarnings(try(eval(call.noV, envir = mfExt), 
-            silent = TRUE))
-        L1 <- if (class(obj1)[1] == "try-error") 
-            L0 + 10
-        else eval(parse(text = fn.obj), list(x = obj1))
-        use.k <- k <- 1
-        L1.k <- NULL
-        L1.k[length(L1.k) + 1] <- L1
-        while (L1 > L0) {
-            k <- k + 1
-            use.k <- if (useExp.k) 
-                2^(k - 1)
-            else k
-            psi <- psi.old + (gamma.c/beta.c)/(use.k * h)
-            if (!is.null(digits)) 
-                psi <- round(psi, digits)
-            PSI <- matrix(rep(psi, rep(n, length(psi))), ncol = length(psi))
-            U <- (Z - PSI) * (Z > PSI)
-            if (pow[1] != 1) 
-                U <- U^pow[1]
-            for (i in 1:ncol(U)) mfExt[nomiU[i]] <- U[, i]
-            obj1 <- suppressWarnings(try(eval(call.noV, envir = mfExt), 
-                silent = TRUE))
-            L1 <- if (class(obj1)[1] == "try-error") 
-                L0 + 10
-            else eval(parse(text = fn.obj), list(x = obj1))
-            L1.k[length(L1.k) + 1] <- L1
-            if (1/(use.k * h) < min.step) {
-                break
-            }
+        #if (pow[1] != 1) U <- U^pow[1]
+
+        if(is.null(opz$mydesign)){
+          for(i in 1:ncol(U)) mfExt[nomiU[i]] <- U[, i]
+        } else {
+          for(i in 1:ncol(U))  mydesign$variables[nomiU[i]] <- U[, i]
         }
+        
+
+        # obj1 <- suppressWarnings(try(eval(call.noV, envir = mfExt), silent = TRUE))
+        # L1 <- if (class(obj1)[1] == "try-error") 
+        #     L0 + 10
+        # else eval(parse(text = fn.obj), list(x = obj1))
+        # use.k <- k <- 1
+        # L1.k <- NULL
+        # L1.k[length(L1.k) + 1] <- L1
+        # # while (L1 > L0) {
+        # #     k <- k + 1
+        # #     use.k <- if (useExp.k) 
+        # #         2^(k - 1)
+        # #     else k
+        # #     psi <- psi.old + (gamma.c/beta.c)/(use.k * h)
+        # #     if (!is.null(digits)) 
+        # #         psi <- round(psi, digits)
+        # #     PSI <- matrix(rep(psi, rep(n, length(psi))), ncol = length(psi))
+        # #     U <- (Z - PSI) * (Z > PSI)
+        # #     if (pow[1] != 1) 
+        # #         U <- U^pow[1]
+        # #     if(is.null(opz$mydesign)){
+        # #       for (i in 1:ncol(U)) mfExt[nomiU[i]] <- U[, i]
+        # #       #obj1 <- suppressWarnings(try(eval(call.noV, envir = mfExt), silent = TRUE))
+        # #     } else {
+        # #       for (i in 1:ncol(U)) mydesign$variables[nomiU[i]] <- U[, i]
+        # #       #obj1 <- suppressWarnings(try(eval(call.noV), silent = TRUE))
+        # #     }
+        # #     obj1 <- suppressWarnings(try(eval(call.noV, envir = mfExt), silent = TRUE))
+        # #     L1 <- if (class(obj1)[1] == "try-error") 
+        # #         L0 + 10
+        # #     else eval(parse(text = fn.obj), list(x = obj1))
+        # #     L1.k[length(L1.k) + 1] <- L1
+        # #     if (1/(use.k * h) < min.step) {
+        # #         break
+        # #     }
+        # # }
         if (visual) {
             flush.console()
             cat(paste("iter = ", sprintf("%2.0f", it), "  min.f = ", 
                 sprintf(paste("%", n.intDev0 + 6, ".5f", sep = ""), 
-                  L1), "  k = ", sprintf("%2.0f", k), "  n.psi = ", 
+                  L1), "  k = ", sprintf("%2.3f", use.k), "  n.psi = ", 
                 formatC(length(unlist(psi)), digits = 0, format = "f"), 
                 "  est.psi = ", paste(formatC(unlist(psi), digits = 3, 
                   format = "f"), collapse = "  "), sep = ""), 
@@ -247,6 +320,7 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
             max(abs((psi - psi.old)/psi.old))
         else (L0 - L1)/(abs(L0) + 0.1)
         L0 <- L1
+        #U<-U1
         k.values[length(k.values) + 1] <- use.k
         psi.values[[length(psi.values) + 1]] <- psi
         dev.values[length(dev.values) + 1] <- L0
@@ -255,11 +329,10 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
         id.psi.ok <- id.psi.in & id.psi.far
         if (!all(id.psi.ok)) {
             if (fix.npsi) {
-                #psi <- adj.psi(psi, limZ)
-                #id.psi.far <- far.psi(Z, PSI, id.psi.group, TRUE)
-                psi <- psi * ifelse(id.psi.far, 1, 0.9)
-                PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
-                id.psi.changed[it]<-TRUE
+              #psi <- psi * ifelse(id.psi.far, 1, 0.9)
+              psi <- psi * ifelse(id.psi.far, 1, attr(id.psi.far, "factor"))  
+              PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
+              id.psi.changed[it]<-TRUE
             }
             else {
                 Z <- Z[, id.psi.ok, drop = FALSE]
@@ -282,10 +355,11 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
             id.warn <- TRUE
             break
         }
-    }
-    if (id.warn)  warning(paste("max number of iterations (", it, ") attained", sep = ""), call. = FALSE)
+    } #end while
+    ###############################################################################################
     if(id.psi.changed[length(id.psi.changed)]) warning(paste("Some psi (", (1:length(psi))[!id.psi.far],
                                                              ") changed after the last iter.",sep=""), call. = FALSE)
+    if (id.warn)  warning(paste("max number of iterations (", it, ") attained", sep = ""), call. = FALSE)
     attr(psi.values, "dev") <- dev.values
     attr(psi.values, "k") <- k.values
     psi <- unlist(tapply(psi, id.psi.group, sort))
@@ -293,22 +367,25 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
     names.coef <- names(coef(obj))
     PSI.old <- PSI
     PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
-    if(sd(PSI-PSI.old)>0 || id.psi.changed[length(id.psi.changed)]){
+    #if(sd(PSI-PSI.old)>0 || id.psi.changed[length(id.psi.changed)]){
         U <- (Z - PSI) * (Z > PSI)
         colnames(U) <- paste("U", 1:ncol(U), sep = "")
         V <- -(Z > PSI)
         colnames(V) <- paste("V", 1:ncol(V), sep = "")
-        for (i in 1:n.psi1) {
+        if(is.null(opz$mydesign)){
+          for (i in 1:n.psi1) {
             mfExt[nomiU[i]] <- U[, i]
             mfExt[nomiV[i]] <- V[, i]
+          }
+        } else {
+          mydesign$variables[nomiU[i]] <- U[, i]
+          mydesign$variables[nomiV[i]] <- V[, i]
         }
-        obj <- suppressWarnings(try(eval(call.noV, envir = mfExt), 
-            silent = TRUE))
+        obj <- suppressWarnings(try(eval(call.noV, envir = mfExt), silent = TRUE))
         L1 <- eval(parse(text = fn.obj), list(x = obj))
-    }
-    else {
-        obj <- obj1
-    }
+    #} else {
+    #    obj <- obj1
+    #}
     nomeCoef <- grep("coef", names(obj), value = TRUE)
     if(length(nomeCoef)==0){
         nomeCoef <- grep("estimate", names(obj), value = TRUE) 
@@ -331,7 +408,7 @@ seg.def.fit<-function (obj, Z, PSI, mfExt, opz, return.all.sol = FALSE) {
     obj <- list(obj = obj, it = it, psi = psi, psi.values = psi.values, 
         U = U, V = V, rangeZ = rangeZ, epsilon = epsilon, nomiOK = nomiOK, 
         SumSquares.no.gap = L1, id.psi.group = id.psi.group, 
-        id.warn = id.warn, nomiV = nomiV, nomiU = nomiU, mfExt = mfExt)
+        id.warn = id.warn, nomiV = nomiV, nomiU = nomiU, mfExt = mfExt, mydesign=mydesign)
     if (vincoli) {
         obj$R <- R
         obj$R.noV <- R.noV
