@@ -4,36 +4,38 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     #-----------------
     #NB iniziato a modificare il 28/01 (dopo l'invio della 1.4-0)
     
-    search.min <- function(h, psi, psi.old, X, y, w, offs) {#DUBBIO: Ma fam, eta0, L0 e maxit.glm devo passarli come argom,enti o li trova?
+    search.min <- function(h, psi, psi.old, X, y, w, offs) {
         psi.ok<- psi*h + psi.old*(1-h)
-        PSI <- matrix(rep(psi.ok, rep(n, length(psi.ok))), ncol = length(psi.ok))
+        #PSI <- matrix(rep(psi.ok, rep(n, length(psi.ok))), ncol = length(psi.ok))
+        PSI <- matrix(psi.ok, nrow=n, ncol = length(psi.ok), byrow = TRUE)
         U1 <- (Z - PSI) * (Z > PSI)
         #if (pow[1] != 1) U1 <- U1^pow[1]
         obj1 <- try(suppressWarnings(glm.fit(x = cbind(X, U1), y = y, offset = offs,
-                                    weights = w, family = fam, control = glm.control(maxit = maxit.glm), etastart = eta0)),
+                                    weights = w, family = fam, control = glm.control(maxit = maxit.glm1[it]), etastart = eta0)),
                                     silent = TRUE)
         L1 <- if (class(obj1)[1] == "try-error") L0 + 10 else obj1$dev
+        attr(L1, "eta") <- obj1$linear.predictor
         L1
     }
     #----------------
-    est.k<-function(x1,y1,L0){
-        ax<-log(x1)
-        .x<-cbind(1,ax,ax^2)
-        b<-drop(solve(crossprod(.x),crossprod(.x,y1)))
-        const<-b[1]-L0
-        DD<-sqrt(b[2]^2-4*const*b[3])
-        kk<-exp((-b[2]+ DD) /(2*b[3]))
-        return(round(kk))
-        
-        #  ff<-function(xx) b[1]+b[2]*xx + b[3]*xx^2+ L0
-        #  a<-uniroot(ff, c(log(x[4]), 3.4))
-    }
-    #-----------------
-    dpmax<-function(x,y,pow=1){
-        #deriv pmax
-        if(pow==1) -(x>y) #ifelse(x>y, -1, 0)
-        else -pow*((x-y)*(x>y))^(pow-1)#-pow*pmax(x-y,0)^(pow-1)
-    }
+    # est.k<-function(x1,y1,L0){
+    #     ax<-log(x1)
+    #     .x<-cbind(1,ax,ax^2)
+    #     b<-drop(solve(crossprod(.x),crossprod(.x,y1)))
+    #     const<-b[1]-L0
+    #     DD<-sqrt(b[2]^2-4*const*b[3])
+    #     kk<-exp((-b[2]+ DD) /(2*b[3]))
+    #     return(round(kk))
+    #     
+    #     #  ff<-function(xx) b[1]+b[2]*xx + b[3]*xx^2+ L0
+    #     #  a<-uniroot(ff, c(log(x[4]), 3.4))
+    # }
+    # #-----------------
+    # dpmax<-function(x,y,pow=1){
+    #     #deriv pmax
+    #     if(pow==1) -(x>y) #ifelse(x>y, -1, 0)
+    #     else -pow*((x-y)*(x>y))^(pow-1)#-pow*pmax(x-y,0)^(pow-1)
+    # }
     #--------------------
     in.psi<-function(LIM, PSI, ret.id=TRUE){
         #check if psi is inside the range
@@ -58,12 +60,13 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
         for(i in 1:nSeg){
             if(length(nj[[i]])!=npsij[i]+1) nj[[i]]<- tabulate(rowSums((Z>=PSI)[,id.psi.group==i,drop=FALSE])+1)
             id.ok<-(nj[[i]] >= 2)
-            id.far.ok[[i]] <- id.ok[-length(id.ok)] & id.ok[-1] #esattamente uguale al numero di psi del gruppo i
+            id.far.ok[[i]] <- id.ok[-length(id.ok)] #& id.ok[-1] #esattamente uguale al numero di psi del gruppo i
             ff[[i]]<-ifelse(diff(nj[[i]])>0, 1/fc, fc)
         }
         id.far.ok<-unlist(id.far.ok)
         ff<-unlist(ff)
-        if(!ret.id) {return(all(id.far.ok))
+        if(!ret.id) {
+          return(all(id.far.ok))
         } else {
             attr(id.far.ok,"factor") <- ff
             return(id.far.ok) 
@@ -79,14 +82,19 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     #--------------
     n<- if(is.matrix(y)) nrow(y) else length(y)
     min.step<-opz$min.step
-    rangeZ <- apply(Z, 2, range)
+    #rangeZ <- apply(Z, 2, range)
     alpha<-opz$alpha
-    limZ <- apply(Z, 2, quantile, names=FALSE, probs=c(alpha[1],alpha[2]))
+    rangeZ <- if(is.null(opz$rangeZ)) apply(Z, 2, range) else opz$rangeZ
+    limZ <- if(is.null(opz$limZ)) apply(Z, 2, quantile, names=FALSE, probs=c(alpha[1],alpha[2])) else opz$limZ
+    
     psi<-PSI[1,]
-    psi<-adj.psi(psi, limZ)
-    PSI<-matrix(psi,nrow=n, ncol=ncol(PSI), byrow=TRUE)
+    
+    #browser()
+    
+    psi<- adj.psi(psi, limZ)
+    PSI<- matrix(psi,nrow=n, ncol=ncol(PSI), byrow=TRUE)
     id.psi.group<-opz$id.psi.group
-    conv.psi<-opz$conv.psi 
+    #conv.psi<-opz$conv.psi 
     digits<-opz$digits
     pow<-opz$pow
     nomiOK<-opz$nomiOK
@@ -104,7 +112,7 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     epsilon <- 10
     k.values<-dev.values<- NULL
     psi.values <-list()
-    psi.values[[length(psi.values) + 1]] <- NA
+    #psi.values[[length(psi.values) + 1]] <- NA
     #id.psi.ok<-rep(TRUE, length(psi))
     sel.col.XREG<-unique(sapply(colnames(XREG), function(x)match(x,colnames(XREG))))
     if(is.numeric(sel.col.XREG))XREG<-XREG[,sel.col.XREG,drop=FALSE] #elimina le ripetizioni, ad es. le due intercette..
@@ -117,42 +125,71 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     if(!far.psi(Z,PSI,id.psi.group,FALSE)) 
       stop("psi starting values too close each other or at the boundaries. Please change them (e.g. set 'quant=TRUE' 
           in seg.control()), or decrease their number.", call. = FALSE)
-    n.psi1<-ncol(Z)
+    n.psi<-n.psi1<-ncol(Z)
     #==============================================
-    U <- ((Z-PSI)*(Z>PSI)) #pmax((Z - PSI), 0)^pow[1]
-    if(pow[1]!=1) U<-U^pow[1]
-    obj0 <- suppressWarnings(glm.fit(x = cbind(XREG, U), y = y, offset = offs,
-                                     weights = w, family = fam, control = glm.control(maxit = maxit.glm), etastart = eta0))
-    eta0<- obj0$linear.predictors
-    L0<- obj0$dev
-    
+    V <- (Z>PSI)
+    U <- (Z-PSI)*V #pmax((Z - PSI), 0)^pow[1]
+    V<- -V
+    #if(pow[1]!=1) U<-U^pow[1]
     
     if(it.max==0){
+      obj <- suppressWarnings(glm.fit(x = cbind(XREG, U), y = y, offset = offs,
+                                      weights = w, family = fam, etastart = eta0))
       colnames(U) <- paste("U", 1:ncol(U), sep = "")
       V <- -(Z > PSI)
       colnames(V) <- paste("V", 1:ncol(V), sep = "")
-      obj <- obj0 #lm.wfit(x = cbind(XREG, U), y = y, w = w, offset = offs)
-      L1 <- L0
       obj$coefficients <- c(obj$coefficients, rep(0, ncol(V)))
-      #names(obj$coefficients) <- names.coef
       obj$epsilon <- epsilon
       obj$it <- it
       obj <- list(obj = obj, it = it, psi = psi, psi.values = psi.values, 
                   U = U, V = V, rangeZ = rangeZ, epsilon = epsilon, nomiOK = nomiOK, 
-                  dev.no.gap = L1, id.psi.group = id.psi.group, 
+                  dev.no.gap = obj$dev, id.psi.group = id.psi.group, idU=ncol(XREG)+1:(length(psi)),
                   id.warn = TRUE)
       return(obj)
     }
     
+    
+    #browser()
+    
+    if(!opz$usesegreg){
+      dev.values[length(dev.values) + 1] <- opz$dev0 #modello senza psi 
+      psi.values[[length(psi.values) + 1]] <- NA #nessun psi 
+    }
+    
+    if(is.null(opz$fit.psi0)){
+      obj <- suppressWarnings(glm.fit(x = cbind(XREG, U), y = y, offset = offs,
+                                      weights = w, family = fam, etastart = opz$eta0))
+      L0 <- obj$dev
+      eta0 <- obj$linear.predictors
+    } else {
+      L0   <- opz$fit.psi0$L0
+      eta0 <- opz$fit.psi0$eta0
+    }
+    
     n.intDev0<-nchar(strsplit(as.character(L0),"\\.")[[1]][1])
-    dev.values[length(dev.values) + 1] <- opz$dev0 #del modello iniziale (senza psi)
+    
+    
     dev.values[length(dev.values) + 1] <- L0 #modello con psi iniziali
     psi.values[[length(psi.values) + 1]] <- psi #psi iniziali
+    
+    #browser() 
+    #sum(opz$fam$dev.resids(y,mean(y),y))
+    
+    if(is.null(maxit.glm)){
+      Nboot <- if(is.null(opz$Nboot)) 0 else opz$Nboot
+      maxit.glm1 <- rep(1:it.max + Nboot, 1:it.max+1) #2*rep(1:it.max, 1:it.max)
+      maxit.glm1 <- pmin(maxit.glm1, 25)
+    } else {
+      maxit.glm1 <- rep(maxit.glm, it.max)
+    }
+    #
+
     #==============================================
-    #browser()
+    
     if (visual) {
         cat(paste("iter = ", sprintf("%2.0f",0),
-                  "  dev = ", sprintf(paste("%", n.intDev0+6, ".5f",sep=""), L0), #formatC(L1,width=8, digits=5,format="f"), #era format="fg" 
+                  #"  dev = ", sprintf(paste("%", n.intDev0+6, ".5f",sep=""), L0), #formatC(L1,width=8, digits=5,format="f"), #era format="fg" 
+                  "  dev = ",  sprintf("%1.5f", as.numeric(strsplit(format(L0, scientific=TRUE), "e")[[1]][1])),
                   "  k = ", sprintf("%5.0f", NA),
                   "  n.psi = ",formatC(length(unlist(psi)),digits=0,format="f"), 
                   "  ini.psi = ",paste(formatC(unlist(psi),digits=3,format="f"), collapse="  "), #sprintf('%.2f',x)
@@ -161,27 +198,42 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     #==============================================  
     id.warn <- FALSE
     id.psi.changed<-rep(FALSE, it.max)
+    tolOp <-if(is.null(opz$tol.opt)) seq(.001, .Machine$double.eps^0.25, l=it.max) else rep(opz$tol.opt, it.max)
+    idU <- ncol(XREG)+ 1:n.psi
+    idV <- 1:n.psi + max(idU)
+    #browser()
+    
     while (abs(epsilon) > toll) {
         it<-it+1
         n.psi0 <- n.psi1
         n.psi1 <- ncol(Z)
         if(n.psi1!=n.psi0){
-            U <- ((Z-PSI)*(Z>PSI)) #pmax((Z - PSI), 0)^pow[1]
-            if(pow[1]!=1) U<-U^pow[1]
-            obj0 <- suppressWarnings(glm.fit(x = cbind(XREG, U), y = y, offset = offs,
-                                             weights = w, family = fam, control = glm.control(maxit = maxit.glm), etastart = eta0))
-            eta0<-obj0$linear.predictors
-            L0< - obj0$dev
+          V<- (Z > PSI)   
+          U <- (Z - PSI) * V
+          V<- -V
+          idU <- ncol(XREG)+ 1:n.psi1
+          idV <- 1:n.psi1 + max(idU)
+          
+          #if(pow[1]!=1) U<-U^pow[1]
+          obj0 <- suppressWarnings(glm.fit(x = cbind(XREG, U), y = y, offset = offs,
+                                             weights = w, family = fam, control = glm.control(maxit = maxit.glm1[it]), etastart = eta0))
+          eta0<-obj0$linear.predictors
+          L0< - obj0$dev
         }
-        V <- dpmax(Z,PSI,pow=pow[2])# ifelse((Z > PSI), -1, 0)
+        #        V <- dpmax(Z,PSI,pow=pow[2])# ifelse((Z > PSI), -1, 0)
         X <- cbind(XREG, U, V)
-        rownames(X) <- NULL
-        colnames(X)[(ncol(XREG) + 1):ncol(X)] <- c(paste("U", 1:ncol(U), sep = ""), paste("V", 1:ncol(V), sep = ""))
+        #rownames(X) <- NULL
+        #colnames(X)[(ncol(XREG) + 1):ncol(X)] <- c(paste("U", 1:ncol(U), sep = ""), paste("V", 1:ncol(V), sep = ""))
         obj <- suppressWarnings(glm.fit(X, y, offset = offs, weights = w, family = fam, 
-                                        control = glm.control(maxit = maxit.glm), etastart = eta0))
+                                        control = glm.control(maxit = maxit.glm1[it]), etastart = eta0))
         eta0<-obj$linear.predictors
-        beta.c <- coef(obj)[paste("U", 1:ncol(U), sep = "")]
-        gamma.c <- coef(obj)[paste("V", 1:ncol(V), sep = "")]
+        #beta.c <- coef(obj)[paste("U", 1:ncol(U), sep = "")] #obj$coefficients[idU]
+        #gamma.c <- coef(obj)[paste("V", 1:ncol(V), sep = "")] #obj$coefficients[idV]
+        beta.c <- obj$coefficients[idU]
+        gamma.c <- obj$coefficients[idV]
+        
+        
+        #if(it==2) browser()
         
         if(any(is.na(c(beta.c, gamma.c)))){
             if(fix.npsi)  {
@@ -210,11 +262,12 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
         psi <- psi.old + hh*gamma.c/beta.c
         psi<- adj.psi(psi, limZ) #limZ or rangeZ???
         
-        psi<-unlist(tapply(psi, opz$id.psi.group, sort), use.names =FALSE)
+        psi<-unlist(tapply(psi, id.psi.group, sort), use.names =FALSE)
         
         #############################aggiusta la stima di psi (nel range.. dopo in limZ)
         #DIREZIONE
-        a<-optimize(search.min, c(0,1), psi=psi, psi.old=psi.old, X=XREG, y=y, w=w, offs=offs) #DUBBIO: Ma fam, eta0, L0 e maxit.glm devo passarli come argom,enti o li trova?
+        #if(it==2) browser()
+        a <-optimize(search.min, c(0,1), psi=psi, psi.old=psi.old, X=XREG, y=y, w=w, offs=offs, tol=tolOp[it]) 
         k.values[length(k.values) + 1] <- use.k <- a$minimum
         L1<- a$objective
         #L1.k[length(L1.k) + 1] <- L1<- a$objective
@@ -223,10 +276,13 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
         psi<- adj.psi(psi, limZ)
 
         if(!is.null(digits)) psi<-round(psi, digits)
-        PSI <- matrix(rep(psi, rep(n, length(psi))), ncol = length(psi))
+        #PSI <- matrix(rep(psi, rep(n, length(psi))), ncol = length(psi))
+        PSI <- matrix(psi, n, length(psi), byrow=TRUE)
         #--modello con il nuovo psi
-        U1<-(Z-PSI)*(Z>PSI)
-        
+        V <- (Z>PSI)
+        U<-(Z-PSI)*V
+        V<- -V
+        eta0<- attr(a$objective, "eta")
         
         #if(pow[1]!=1) U1<-U1^pow[1]
         # obj1 <- try(suppressWarnings(glm.fit(cbind(XREG, U1), y = y, offset = offs,
@@ -258,16 +314,16 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
         if (visual) {
             flush.console()
             cat(paste("iter = ", sprintf("%2.0f",it),
-                      "  dev = ", sprintf(paste("%", n.intDev0+6, ".5f",sep=""), L1), #formatC(L1,width=8, digits=5,format="f"), #era format="fg" 
+                      #"  dev = ", sprintf(paste("%", n.intDev0+6, ".5f",sep=""), L1), #formatC(L1,width=8, digits=5,format="f"), #era format="fg" 
+                      "  dev = ",  sprintf("%1.5f", as.numeric(strsplit(format(L1, scientific=TRUE), "e")[[1]][1])),
                       "  k = ", sprintf("%2.3f", use.k),
                       "  n.psi = ",formatC(length(unlist(psi)),digits=0,format="f"), 
                       "  est.psi = ",paste(formatC(unlist(psi),digits=3,format="f"), collapse="  "), #sprintf('%.2f',x)
                       sep=""), "\n")
         }
-        epsilon <- if(conv.psi) max(abs((psi -psi.old)/psi.old)) else (L0 - L1)/(abs(L0) + 0.1) 
+        epsilon <- (L0 - L1)/(abs(L0) + 0.1) 
         L0<-L1
-        U <-U1
-        
+
         k.values[length(k.values)+1]<-use.k
         psi.values[[length(psi.values) + 1]] <- psi
         dev.values[length(dev.values) + 1] <- L0
@@ -281,8 +337,9 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
             if(fix.npsi){
                 #psi<-adj.psi(psi, limZ) #within range!!!
                 #id.psi.far<-far.psi(Z, PSI, id.psi.group, TRUE)
-                psi<-psi*ifelse(id.psi.far,1,.9) 
-                PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
+                #psi<-psi*ifelse(id.psi.far,1,.9) 
+              psi <- psi * ifelse(id.psi.far, 1, attr(id.psi.far, "factor"))  
+              PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
                 id.psi.changed[it]<-TRUE
                     } else {
                 Z<-Z[, id.psi.ok, drop=FALSE]
@@ -319,30 +376,25 @@ seg.glm.fit<-function(y,XREG,Z,PSI,w,offs,opz,return.all.sol=FALSE){
     names.coef<-names(obj$coefficients) #obj e' quello vecchio che include U1,.. V1,...
     
     PSI.old<-PSI
-    PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
-    
+    #PSI <- matrix(rep(psi, rep(nrow(Z), length(psi))), ncol = length(psi))
+    PSI <- matrix(psi, nrow=n, ncol = length(psi), byrow=TRUE)
     #U e V possono essere cambiati (rimozione/ordinamento psi.. ) per cui si deve ricalcolare il tutto, altrimenti sarebbe uguale a U1 e obj1
     #if(sd(PSI-PSI.old)>0 || id.psi.changed[length(id.psi.changed)]){
-        U <- (Z-PSI)*(Z>PSI)
-        colnames(U)<-paste("U", 1:ncol(U), sep = "")
-        V <- -(Z>PSI)
-        colnames(V)<-paste("V", 1:ncol(V), sep = "")
-#        X <- cbind(XREG, U, V)
-#        rownames(X) <- NULL
-        obj <- try(suppressWarnings(glm.fit(cbind(XREG, U), y = y, offset = offs,
-                    weights = w, family = fam, control = glm.control(maxit = maxit.glm), etastart = eta0)), 
+    U <- (Z-PSI)*(Z>PSI)
+    #colnames(U)<-paste("U", 1:ncol(U), sep = "")
+    V <- -(Z>PSI)
+    #colnames(V)<-paste("V", 1:ncol(V), sep = "")
+    obj <- try(suppressWarnings(glm.fit(cbind(XREG, U), y = y, offset = offs,
+                    weights = w, family = fam, control = glm.control(maxit = maxit.glm1[it]), etastart = eta0)), 
                     silent = TRUE)
-        L1<- obj$dev
-        #} else {
-        #obj<-obj1
-        #}
+    L1<- obj$dev
     obj$coefficients<-c(obj$coefficients, rep(0,ncol(V)))
     names(obj$coefficients)<-names.coef
-    obj$epsilon <- epsilon
-    obj$it <- it
-    
-    obj<-list(obj=obj,it=it,psi=psi, psi.values=psi.values, U=U,V=V,rangeZ=rangeZ,
-              epsilon=epsilon,nomiOK=nomiOK, dev.no.gap=L1, id.psi.group=id.psi.group,
-              id.warn=id.warn) #inserire id.psi.ok?
+
+    #browser()
+    obj$epsilon <- as.numeric(epsilon)
+    obj<-list(obj=obj,it=it, psi=psi, psi.values=psi.values, U=U,V=V,rangeZ=rangeZ, 
+              epsilon=as.numeric(epsilon),nomiOK=nomiOK, dev.no.gap=L1, id.psi.group=id.psi.group, idU=ncol(XREG)+1:(length(psi)), 
+              id.warn=id.warn, eta0=eta0) #inserire id.psi.ok?
     return(obj)
 }
