@@ -10,6 +10,20 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
   #Allora considerando seggrowth() qua ci dovrebbero essere problemi in nel predict..
   
   #=================================
+  # `[.withAttributes` <- function(r, i) {
+  #   subset <- NextMethod()
+  #   attr(subset, "nomeBy") <- attr(r, "nomeBy")
+  #   attr(subset, "nomeX") <- attr(r, "nomeX")
+  #   attr(subset, "psi") <- attr(r, "psi")
+  #   attr(subset, "npsi") <- attr(r, "npsi")
+  #   attr(subset, "est") <- attr(r, "est")
+  #   attr(subset, "R") <- attr(r, "R")
+  #   attr(subset, "fix.psi") <- attr(r, "fix.psi")
+  #   attr(subset, "f.x") <- attr(r, "f.x")
+  #   attr(subset, "by") <- attr(r, "by")
+  #   attr(subset, "levelsBy") <- attr(r, "levelsBy")
+  #   subset
+  # }
   
     build.all.psi<-function(psi, fixed.psi){
     all.names.psi<-union(names(psi),names(fixed.psi))
@@ -102,14 +116,20 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
     id.ps<-attr(tf,"specials")$seg #posizione nel modelframe; vettore se ci sono piu' termini..include y ma non da interc
 
     #browser()
+    
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "weights",  "na.action"), names(mf), 0L) #"offset",
     mf <- mf[c(1, m)]
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
     names(mf)[2]<-"formula" #serve se NON hai usato "formula"
+    
+    #browser()
+    
     mf <- eval(mf, parent.frame())
     
+    #browser()
+    n<-nrow(mf)
     mt <- attr(mf, "terms")
     intercMt<-attr(mt,"intercept")
     interc<-intercMt==1
@@ -124,21 +144,25 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
         Y <- eval(parse(text=transf), list(y=Y))
         transf.inv<-splinefun(Y, Y.orig, ties=min, method="monoH.FC")
     }
+    if(is.null(alpha)) alpha<- max(.05, 1/length(Y))
+    if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
     
+    #browser()
     .xlivelli<-.getXlevels(mt, mf) 
     weights <- as.vector(model.weights(mf))
     if(!is.null(weights) && !is.numeric(weights)) stop("'weights' must be a numeric vector")
     if(!is.null(weights) && any(weights < 0)) stop("negative weights not allowed")
     offs <- as.vector(model.offset(mf))
+    
     #browser() #funziona sia nella formula che come argomento?
     
     testo.ps<-names(mf)[id.ps]
     nomiCoefUNPEN<-names(mf)[-c(1,id.ps)]
-    X <- if(!is.empty.model(mt)){
-        model.matrix(mt, mf, contrasts) 
-      } else {stop("error in the design matrix")}#matrix(, NROW(Y), 0L)
-    attrContr<-attr(X, "contrasts")
-    n<-nrow(X)
+    # X <- if(!is.empty.model(mt)){
+    #     model.matrix(mt, mf, contrasts) 
+    #   } else {stop("error in the design matrix")}#matrix(, NROW(Y), 0L)
+    # attrContr<-attr(X, "contrasts")
+    # n<-nrow(X)
     
     
     #browser()
@@ -197,9 +221,10 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
     levelsBy <- lapply(l,function(xx) attr(xx,"levelsBy"))
     
     #browser()
+    
     if(all(sapply(levelsBy, is.null)) && (length(npsiList)!=length(nomiPS))) stop(" 'npsi' is not correctly specified")
     
-    rangeSmooth<-mVariabili<-NULL
+    limZ<-rangeSmooth<-mVariabili<-NULL
 		#se ci sono termini ps()+ps(..,by) il nome delle variabili smooth vengono cambiati per aggiungere la variabile by
 		nomiPS.orig <- nomiPS
 		  
@@ -220,19 +245,22 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
     #BFixed<-BB<-Bderiv
     
     
-    
+    #browser()
     
     for(j in 1:length(mVariabili)) {
       if(nomiBy[j]=="NULL"){ # se usuale termine seg()
         nomiBy.j <- NULL
         variabileSmooth<- c(mVariabili[[j]]) #c() converte le matrici in vettori, drop() no..!
-        variabileSmooth<- attr( mVariabili[[j]], "f.x")(variabileSmooth)
+        #variabileSmooth<- attr(mVariabili[[j]], "f.x")(variabileSmooth)
+        variabileSmooth<- fxList[[j]](variabileSmooth)
         #for(jj in c("nomeX", "psi", "npsi", "f.x", "nomeBy")) attr(variabileSmooth,jj)<-NULL
         B[[j]]<- variabileSmooth
         rangeSmooth[[j]] <- range(variabileSmooth)
+        limZ[[j]] <- quantile(variabileSmooth, names=FALSE, probs=c(alpha[1],alpha[2]))
         nomiPS.ps.int.list[[j]]<- nomiPS[j]
         
       } else { #se ci sono termini by
+        #browser()
         if(is.null(levelsBy[[j]])){ #se e' vc con variabile continua
           stop(" 'by' in seg(), if provided, should be a factor")
           #B[[j]] <-variabileBy*B[[j]]
@@ -240,11 +268,12 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
         } else {#se e' VC con variabile categoriale
           nomiBy.j <- nomiBy[j]
           variabileSmooth<- mVariabili[[j]][,-ncol(mVariabili[[j]]),drop=TRUE] 
-          variabileSmooth<- attr( mVariabili[[j]], "f.x")(variabileSmooth)
+          variabileSmooth<- fxList[[j]](variabileSmooth)
           variabileBy<- mVariabili[[j]][, ncol(mVariabili[[j]]),drop=TRUE]
           M<-model.matrix(~0+factor(variabileBy))
           B[[j]]<- lapply(1:ncol(M), function(.x) M[,.x]*variabileSmooth)
           rangeSmooth[[j]] <- lapply(B[[j]], function(.x) range(.x[.x!=0]))
+          limZ[[j]] <- lapply(B[[j]], function(.x) quantile(.x[.x!=0], names=FALSE, probs=c(alpha[1],alpha[2])))
           #browser()
           cond1 <- is.list(psiList[[j]])
           cond2 <- length(names(psiList[[j]]))==length(levelsBy[[j]])
@@ -259,6 +288,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
     } #end for(j in 1:length(mVariabili))
     
     #browser()
+    #sapply(limZ[[1]], cbind)
     
     repl<-pmax(sapply(B,length)*sapply(B,is.list),1)
     for(i in 1:length(npsiList)){
@@ -272,13 +302,21 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       }
     }
     npsiList <- unlist(npsiList)
-      
+    
+    #browser()
+    
     if(!any(sapply(psiList,is.list))) psiList <- rep(psiList, repl)
     if(!any(sapply(estList,is.list))) estList <- rep(estList, repl)
     if(!any(sapply(RList,is.list))) RList <- rep(RList, repl)
+    if(!any(sapply(fixpsiList,is.list))) fixpsiList<- rep(fixpsiList, repl)
+    
+    
     nomiPS.orig <- rep(nomiPS.orig, repl)
     Bfix <- rep(Bfix, repl)
-    fixpsiList<- rep(fixpsiList, repl)
+    
+    
+    
+    
     while(any(sapply(B,is.list))){
         id.vc<-which((sapply(B, is.list)))[1]
         nc<-length(B[[id.vc]])
@@ -295,20 +333,27 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
         rangeSmooth <- append(rangeSmooth, rangeSmooth[[id.vc]], after = id.vc-1)
         rangeSmooth[[id.vc+nc]]<-NULL
         
+        limZ <- append(limZ, limZ[[id.vc]], after = id.vc-1)
+        limZ[[id.vc+nc]]<-NULL
+        
         estList <- append(estList, estList[[id.vc]], after = id.vc-1)
         estList[[id.vc+nc]]<-NULL
         
         RList <- append(RList, RList[[id.vc]], after = id.vc-1)
         RList[[id.vc+nc]]<-NULL
         
+        fixpsiList <- append(fixpsiList, fixpsiList[[id.vc]], after = id.vc-1)
+        fixpsiList[[id.vc+nc]]<-NULL
+        
+        
         #se la lista contiene solo NULL, non funziona...
         #penMatrixList <- append(penMatrixList, penMatrixList[[id.vc]], after = id.vc-1)
         #penMatrixList[[id.vc+nc]]<-NULL
     }
     
-      
       #if(!all(sapply(estList,check.estPsi))) stop(" 'est' is misspecified in one or more seg() term")
-      
+    #browser()
+    
     nomiTerminiSEG<-nomiCoefPSI <-NULL
     nomiPS.ps.unlist.seg <- unlist(nomiPS.ps.list)
     nomiPS.ps.unlist <- sub("[)]", "", sub("seg[(]", "",nomiPS.ps.unlist.seg ))
@@ -346,22 +391,25 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
     if(any(sapply(estList, is.list))) stop(" One or more 'est' components misspecified")
     
     npsiList1<-id.contrR <- rep(NA, length(B))
+    psiListE<-psiListQ<-psiList
     for(j in 1:length(B)){
         #K<- npsiList[j]
         K <- if(!is.na(npsiList[nomiSeg[j]])) npsiList[nomiSeg[j]] else npsiList[j]
         npsiList1[j]<- K
         if(any(is.na(psiList[[j]]))){ 
-          if(control$quant) {
-            psiList[[j]]<- quantile(B[[j]], prob= seq(0,1,l=K+2)[-c(1,K+2)], names=FALSE)
-          } else {
-            psiList[[j]]<- (min(B[[j]])+ diff(range(B[[j]]))*(1:K)/(K+1))
-          }
+          #if(control$quant) {
+            psiListQ[[j]]<- quantile(B[[j]], prob= seq(0,1,l=K+2)[-c(1,K+2)], names=FALSE)
+          #} else {
+            psiListE[[j]]<- (min(B[[j]])+ diff(range(B[[j]]))*(1:K)/(K+1))
+          #}
         } else {
           K<-npsiList1[j]<-length(psiList[[j]])
         }
         if(!is.null(fixpsiList[[j]])) {
           Bfix[[j]]<- sapply(sort(fixpsiList[[j]]), function(.x) (B[[j]]-.x)*(B[[j]]>.x))
-          colnames(Bfix[[j]])<- paste("U", 1:length(fixpsiList[[j]]),".fixed.",nomiPS.orig[j], sep="")
+          #colnames(Bfix[[j]])<- paste("U", 1:length(fixpsiList[[j]]),".fixed.",nomiPS.orig[j], sep="")
+          colnames(Bfix[[j]])<- paste("U", 1:length(fixpsiList[[j]]),".fixed.",nomiPS.ps.unlist[j], sep="")
+          
         }
       #se per qualche termine ci sono le matrici dei vincoli sulle slope
         #browser()
@@ -375,7 +423,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
             j.ok=match(nomiSeg[j], names(estList), nomatch=0)
             j.ok <-if(j.ok>0) j.ok else j
             if(!any(is.na(estList[[j.ok]]))){
-              if(length(estList[[j.ok]])!=(K+1)) stop(" 'est' is not compatible with 'n.psi' ")
+              if(length(estList[[j.ok]])!=(K+1)) stop(" 'est' is not compatible with 'npsi' ")
               #browser()
               RList[[j]]<-diag(K+1)[,estList[[j.ok]]==1,drop=FALSE]
               id.contrR[j] <-TRUE
@@ -386,8 +434,21 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
           }
     }
     
-      
-      #NB: Poiche' ora psiList include i veri numeri dei psi, i codici vanno rilanciati
+    #browser()
+    
+    if(control$quant) {
+      psiList<-psiListQ
+      initial <- unlist(psiListE)
+      PSI1<- matrix(initial, n, length(initial), byrow = TRUE)
+      } else {
+        psiList<-psiListE
+        initial <- unlist(psiListQ)
+        PSI1<- matrix(initial, n, length(initial), byrow = TRUE)
+      }
+    #Quindi PSI1 e' una matrice di valori inziali di psi.. Vanno usati da seg.lm.fit.boot nel caso in cui i primi PSI
+    #non portino ad adattare un modello
+    
+    #NB: Poiche' ora psiList include i veri numeri dei psi, i codici vanno rilanciati
       for(i in 1:length(B)) {
         nomiCoefPSI[[i]]<- paste(paste("psi",1:length(psiList[[i]]), sep=""), nomiPS.ps.unlist[i], sep=".") ##oppure sep=".psi"
         nomiTerminiSEG[[i]]<-rep(nomiPS.ps.unlist[i], length(psiList[[i]]))
@@ -395,10 +456,8 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       nomiCoefU<-lapply(nomiCoefPSI, function(.x) sub("psi","U",.x )) 
       nomiCoefZ<-lapply(nomiCoefPSI, function(.x) sub("psi","Z",.x ))
 
-
+      npsii <- sapply(psiList,length)
       id.psi.group <- rep(1:length(psiList), sapply(psiList,length))
-      #browser()
-      
       Z<- lapply(1:length(B), function(.x) matrix(B[[.x]], nrow=n, ncol=npsiList1[[.x]]))
       Z<- do.call(cbind,Z)
       colnames(Z) <- unlist(nomiCoefZ)
@@ -418,7 +477,13 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       #nomiInterCateg: "x:g1" "x:g2" "x:g3" ..
       
       #========================================================================================================
-
+      X <- if(!is.empty.model(mt)){
+        model.matrix(mt, mf, contrasts) 
+      } else {stop("error in the design matrix")}#matrix(, NROW(Y), 0L)
+      attrContr<-attr(X, "contrasts")
+      #n<-nrow(X)
+      
+      
       X<- X[, !startsWith(colnames(X),"seg("), drop=FALSE]
 
       idZ <- unlist(tapply(id.psi.group, id.psi.group, function(.x) c(TRUE, rep(FALSE, length(.x)-1))))
@@ -428,7 +493,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       #colnames(Z)<- unlist(nomiCoefPEN)
       initial <- unlist(psiList)
       PSI <- matrix(initial, n, length(initial), byrow = TRUE)
-      
+      #PSI1 di riserva definito sopra.
       #browser()
       
       #NB la matrie del disegno X include in nomi "seg(x)" e non va bene perche' poi da problemi con i 
@@ -447,28 +512,51 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       #X[,nomiPS.orig] <- Z[, unique(colnames(Z)), drop=FALSE]
       #nomiCoefPEN include i nomi le interazioni con i livelli (nel caso vc) e anche del numero dei psi
       #[1] "U1.x" "U2.x" "U1.z"
+      id.noOW <- if(is.null(weights) && is.null(offs)) TRUE else FALSE 
       if(is.null(weights)) weights<-rep(1,n)
+      orig.offs<-offs
       if(is.null(offs)) offs<-rep(0,n)
 
-      invXtX<-Xty<-NULL
-      if(is.null(alpha)) alpha<- max(.05, 1/length(Y))
-      if(length(alpha)==1) alpha<-c(alpha, 1-alpha)
+      #E' realmente necessario assegnare offs e weight con 0 e 1 anche se non servono???
       #browser()
-      opz<-list(toll=toll,h=h,stop.if.error=stop.if.error,dev0=var(Y)*(n-1),visual=visual,it.max=it.max,nomiOK=unlist(nomiCoefU),
-                fam=family, eta0=NULL, maxit.glm=maxit.glm, id.psi.group=id.psi.group, gap=gap,
-                conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step,
-                pow=pow, visualBoot=visualBoot, digits=digits, fc=fc, RList=RList, nomiSeg=nomiSeg, seed=control$seed)
+      
+      
+      limZ <-do.call(cbind, lapply(limZ, function(.x){if(is.list(.x)) do.call(cbind, .x) else cbind(.x)} ))
+      #limZ <-matrix(sapply(1:length(npsii), function(.x) rep(limZ[,.x],npsii[.x])), nrow=2, byrow = FALSE)
+      limZ <- do.call(cbind, lapply(1:length(npsii), function(.x) matrix(limZ[,.x],nrow=2,ncol=npsii[.x])))
+      rangeZ <- do.call(cbind, lapply(1:length(npsii), function(.x) matrix(rangeSmooth[[.x]],nrow=2,ncol=npsii[.x])))
+      colnames(rangeZ)<-rep(names(npsii), npsii) #unlist(nomiTerminiSEG)
+      
+      #browser()
+      
+      #rangeZ<- matrix(sapply(1:length(npsii), function(.x) rep(rangeSmooth[[.x]],npsii[.x])), nrow=2, byrow = FALSE)
+      
+      
+      invXtX<-Xty<-NULL
+      
+      
+      #browser()
+      opz<-list(toll=toll,h=h,stop.if.error=stop.if.error,L0=NULL,visual=visual,it.max=it.max,nomiOK=unlist(nomiCoefU), usesegreg=TRUE,
+                fam=family, eta0=NULL, maxit.glm=maxit.glm, id.psi.group=id.psi.group, gap=gap, limZ=limZ, rangeZ=rangeZ,
+                conv.psi=conv.psi, alpha=alpha, fix.npsi=fix.npsi, min.step=min.step, tol.opt=control$tol.opt,
+                pow=pow, visualBoot=visualBoot, digits=digits, fc=fc, RList=RList, nomiSeg=nomiSeg, 
+                seed=control$seed, min.n=control$min.n, PSI1=PSI1)
+      
       #browser()
       
       if(any(id.contrR)){
         if(fitter0=="lm"){
+          # for(.i in nomiSeg) { #    #poni min(z)=0, cosi solve() in step.lm.fit non ha problemi.
+          #   if(.i %in% colnames(X)) X[,.i]<- X[,.i] - min(X[,.i])
+          # }
+          
           if(n.boot <= 0) {
           obj <- segConstr.lm.fit(Y, X, Z, PSI, weights, offs, opz)
           } else {
             obj <- segConstr.lm.fit.boot(Y, X, Z, PSI, weights, offs, opz, 
                                  n.boot = n.boot, size.boot = size.boot, random = random, 
                                  break.boot = break.boot)
-            seed<- obj$seed
+  #          seed<- obj$seed
           }
           class0<- "lm"
           if(obj$obj$df.residual==0) warning("no residual degrees of freedom (other warnings expected)", call.=FALSE)
@@ -479,12 +567,19 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
             obj <-segConstr.glm.fit.boot(Y, X, Z, PSI, weights, offs, opz, 
                                    n.boot=n.boot, size.boot=size.boot, random=random, 
                                    break.boot=break.boot)
-            seed<- obj$seed
+ #           seed<- obj$seed
           }
           class0<-c("glm","lm")
           }
       } else {
+        
+        
+        #browser()
+        
+        
+        
         if(fitter0=="lm"){
+
           if(n.boot <= 0) {
             obj <- seg.lm.fit(Y, X, Z, PSI, weights, offs, opz)
             } else {
@@ -492,7 +587,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
               obj <- seg.lm.fit.boot(Y, X, Z, PSI, weights, offs, opz, 
                                n.boot = n.boot, size.boot = size.boot, random = random, 
                                break.boot = break.boot)
-              seed<- obj$seed
+#              seed<- obj$seed
           }
           class0<-"lm"
           if(obj$obj$df.residual==0) warning("no residual degrees of freedom (other warnings expected)", call.=FALSE)
@@ -503,14 +598,27 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
             obj <-seg.glm.fit.boot(Y, X, Z, PSI, weights, offs, opz, 
                                 n.boot=n.boot, size.boot=size.boot, random=random, 
                                 break.boot=break.boot)
-            seed<- obj$seed
+   #         seed<- obj$seed
         }
         class0<-c("glm","lm")
         }
       }
       
-      #browser()
-      
+      if(!is.list(obj)){
+        warning("Estimation failed. Too many breakpoints? Returning a (g)lm fit..", call. = FALSE)
+        
+        if(fitter0=="lm"){
+          obj0 <- if(id.noOW) lm.fit(x = X, y = Y) else lm.wfit(x = X, y = Y, w = weights, offset = offs)
+          class(obj0)<-"lm"
+          } else {
+            obj0 <- try(suppressWarnings(glm.fit(X, y = Y, offset = offs,
+                                               weights = weights, family = opz$fam #control = glm.control(maxit = maxit.glm), 
+                                               )), silent = TRUE)
+            class(obj0)<-c("glm", "lm")
+          }
+        return(obj0)
+      }
+      seed<- obj$seed
       
       if(!is.list(obj)){
         warning("No breakpoint estimated", call. = FALSE)
@@ -524,14 +632,15 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       U<-obj$U
       V<-obj$V
       rangeZ<-obj$rangeZ
+      #browser()
       colnames(rangeZ) <- unlist(nomiTerminiSEG)
       it <- obj$it
       epsilon <- obj$epsilon
       id.warn <- obj$id.warn
       k <- length(psi)
       objU <- obj$obj
-      beta.c <- coef(objU)[paste("U", 1:ncol(U), sep = "")]
-      
+      #beta.c <- coef(objU)[paste("U", 1:ncol(U), sep = "")]
+      beta.c <- coef(objU)[obj$idU]
       #browser()
       
       if(any(id.contrR)) {
@@ -559,7 +668,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       colnames(Vxb)<-nomiVxb #<- sub("U","psi", nomiU)
       se.psi<-rep(NA,k)
       if(fitter0=="lm"){
-        objV <- lm.wfit(x = cbind(X, U, Vxb), y = Y, w = weights, offset = offs)
+        objV <- if(id.noOW) lm.fit(x = cbind(X, U, Vxb), y = Y) else lm.wfit(x = cbind(X, U, Vxb), y = Y, w = weights, offset = offs)
         if(var.psi) {
           s2 <- sum(weights*objU$residuals^2)/objV$df.residual
           R <- chol2inv(objV$qr$qr)
@@ -567,8 +676,31 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
         }
       } else {
         objV <- try(suppressWarnings(glm.fit(cbind(X, U, Vxb), y = Y, offset = offs,
-                                           weights = weights, family = opz$fam, control = glm.control(maxit = maxit.glm), 
+                                           weights = weights, family = opz$fam, #control = glm.control(maxit = maxit.glm), 
                                            etastart = objU$linear.predictors)), silent = TRUE)
+        objV$linear.predictors<-objU$linear.predictors
+        objV$deviance<-objU$deviance
+        objV$aic<-objU$aic + 2*ncol(PSI) #k
+        objV$weights<-objU$weights
+        
+        #browser()
+        
+        if (length(offs) && attr(mt, "intercept") > 0L) {
+          #se c'e' un offset devi calcolare la null.deviance (come fa glm())
+          obj0 <- try(suppressWarnings(glm.fit(X[, "(Intercept)", drop = FALSE], y = Y, offset = offs,
+                                               weights = weights, family = opz$fam, #control = glm.control(maxit = maxit.glm), 
+                                               etastart = objU$linear.predictors, intercept=TRUE)), silent = TRUE)
+          
+          # obj0 <- eval(call(if (is.function(method)) "method" else method, 
+          #                   x = X[, "(Intercept)", drop = FALSE], y = Y, mustart = fit$fitted.values, 
+          #                   weights = weights, offset = offset, family = family, 
+          #                   control = control, intercept = TRUE))
+          if (!obj0$converged) warning("fitting to calculate the null deviance did not converge -- increase 'maxit'?")
+          objV$null.deviance <- obj0$deviance
+        }
+        
+        
+        
         if(var.psi) {
           R <- chol2inv(objV$qr$qr)
           s2 <- 1
@@ -577,14 +709,13 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
         }
       }
 
-      #browser()
-      
       objV$fitted.values <- objU$fitted.values
       objV$residuals <- objU$residuals
+      names.coef<-names(objV$coefficients)
       objV$coefficients <- objU$coefficients
       pLin<- ncol(X)
       if(pLin>=1) {
-        names(objV$coefficients) <- c(names(objV$coefficients)[1:pLin], c(nomiU, nomiVxb))
+        names(objV$coefficients) <- c(names.coef[1:pLin], c(nomiU, nomiVxb))
       } else {
         names(objV$coefficients) <- c(nomiU, nomiVxb)
       }
@@ -637,11 +768,7 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       
       #names(mf) <- nomi.mf
       objV$terms <- mt
-      #obj$y<- if(y) Y else NULL
-      
-      #if(!y) fit$y <- NULL
-      #restituire offset?? 
-      objV$y<-Y
+      if(fitter0=="lm") objV$y<-Y #modificato il 6/5/24.. Non c'e' bisogno.. l'ogg restituit da glm.fit() ha gia' la y
       if(x) objV$x <- X
       objV$contrasts <- attrContr  
       objV$xlevels <- .xlivelli 
@@ -689,6 +816,12 @@ segreg <- function(formula, data, subset, weights, na.action, family=lm, control
       objV$id.psi.group<- id.psi.group
       objV$psi[,"Initial"]<-NA
       if(n.boot>0) objV$seed <- seed
+      #browser()
+      
+      #il comando structure() l'ho messo perche' avevo bisogno che anche in mancanza di offset, l'oggetto finale restituisse 
+      # un oggetto con la componente offset NULL. Cosa che non viene fatta con il semplice comando di sotto.. 
+      #objV<- if(id.O) c(objV, offset=offs) else c(objV, offset=NULL)
+      objV<- structure(c(objV, list(offset=orig.offs)))
       class(objV)<-c("segmented", class0)
       objV
     
