@@ -45,6 +45,7 @@ step.num.fit.boot <- function(y, XREG, Z, PSI, w, opz, n.boot=10, size.boot=NULL
     vv<-vv[vv!="0"]
     vv=na.omit(vv[1:5])
     seed <-eval(parse(text=paste(vv, collapse="")))
+    if(is.null(seed)) seed <- 1
     set.seed(seed)
   } else {
     if(is.na(opz$seed)) {
@@ -108,9 +109,10 @@ step.num.fit.boot <- function(y, XREG, Z, PSI, w, opz, n.boot=10, size.boot=NULL
   ###INIZIO BOOT
   alpha<-.1
   corr=1.2
+  n.boot.rev<- 3 #3 o 4?
+  
   for(k in seq(n.boot)){
     ##se gli *ultimi* n.boot.rev valori di ss sono uguali, cambia i psi...
-    n.boot.rev<- 3 #3 o 4?
     diff.selected.ss <- rev(diff(na.omit(all.selected.ss)))
     if(length(diff.selected.ss)>=(n.boot.rev-1) && all(round(diff.selected.ss[1:(n.boot.rev-1)],6)==0)){
       #browser()
@@ -123,6 +125,9 @@ step.num.fit.boot <- function(y, XREG, Z, PSI, w, opz, n.boot=10, size.boot=NULL
       est.psi0 <- adj.psi(est.psi0, limZ)
       #est.psi0<- jitter(est.psi0, amount=min(diff(est.psi0))) 
     }
+    ############################ 25/7/24 #####
+    est.psi0 <- unlist(tapply(est.psi0, opz$id.psi.group, sort))
+    ##########################################
     
     PSI <- matrix(est.psi0, n, ncol = length(est.psi0), byrow=TRUE)
     if(jt) Z<-apply(Z.orig,2,jitter)
